//---------------------------------------------------------------------------
#ifndef MPU6050_H
#define MPU6050_H
//---------------------------------------------------------------------------
#include <Arduino.h>
#include <Wire.h>
#include <Kalman.h>
//---------------------------------------------------------------------------
class TMPU6050
{
private:
	//TTL原始数据计算公式
	#define TTL_Calc_Acc(n)   (n / 32768.0 * 16);		//计算加速度，单位g
	#define TTL_Calc_Gyro(n)  (n / 32768.0 * 2000);		//计算角速度，单位°/s
	#define TTL_Calc_Angle(n) (n / 32768.0 * 180);		//计算角度，单位°
	#define TTL_Calc_Tempe(n) (n / 340.0 + 36.53);		//计算温度，单位°C

	//TTL原始数据结构
	typedef struct TTL_Data_Struct
	{
		uint8_t Head;		//数据包头，固定为0x55
		uint8_t Type;		//数据包类型，0x51：加速度；0x52：角速度；0x53：角度
		int16_t X;			//X轴数据
		int16_t Y;			//Y轴数据
		int16_t Z;			//Z轴数据
		int16_t T;			//温度数据
		uint8_t Checksum;	//校验和，Checksum = Head + Type + X(High) + X(Low) + Y(High) + Y(Low) + Z(High) + Z(Low) + T(High) + T(Low)
	};

	//TTL原始数据
	HardwareSerial* FpSerialInterface;
	uint8_t FnTTL_Buffer[11];
	TTL_Data_Struct FsTTL_Data;

	//I2C读写函数
	uint8_t i2cWrite(const uint8_t nRegisterAddress, const uint8_t nData, const boolean boolSendStop);
	uint8_t i2cWrite(const uint8_t nRegisterAddress, const uint8_t* pnData, const uint8_t nLength, const boolean boolSendStop);
	uint8_t i2cRead(const uint8_t nRegisterAddress, uint8_t* pnData, const uint8_t nBytes);
	uint8_t i2cData();

	//I2C原始数据
	TwoWire* FpI2CInterface;
	const uint8_t FnI2C_Address = 0x68;									//I2C地址
	const uint16_t FnI2C_Timeout = 100;									//超时时间：100微秒
	uint8_t FnI2C_Buffer[14];											//寄存器数据
	double FdI2C_AccX,  FdI2C_AccY,  FdI2C_AccZ,  FdI2C_AccConvert;		//加速度计数据、转换系数
	double FdI2C_GyroX, FdI2C_GyroY, FdI2C_GyroZ, FdI2C_GyroConvert;	//陀螺仪数据、转换系数

	//卡尔曼滤波函数
	void KalmanInit();
	void KalmanFilter();

	//卡尔曼滤波变量
	Kalman FKalmanX, FKalmanY;
	double FdKalmanAngleX, FdKalmanAngleY;
	uint32_t FnKalmanTimer;

	//接口类型，0：串口；1：I2C
	uint8_t FnInterfaceType;
	void Init(const uint8_t nInterfaceType, const double dAccX_Err = 0, const double dAccY_Err = 0, const double dAccZ_Err = 0, const double dGyroX_Err = 0, const double dGyroY_Err = 0, const double dGyroZ_Err = 0, const double dAngleX_Err = 0, const double dAngleY_Err = 0, const double dAngleZ_Err = 0);

	//传感器偏差
	double FdAccX_Err,   FdAccY_Err,   FdAccZ_Err;		//加速度计偏差
	double FdGyroX_Err,  FdGyroY_Err,  FdGyroZ_Err;		//陀螺仪偏差
	double FdAngleX_Err, FdAngleY_Err, FdAngleZ_Err;	//角度偏差

	//返回数据的数据结构
	typedef struct Data_Struct
	{
		double AccX;	//X轴加速度，单位g
		double AccY;	//Y轴加速度，单位g
		double AccZ;	//Z轴加速度，单位g
		double GyroX;	//X轴角速度，单位°/s
		double GyroY;	//Y轴角速度，单位°/s
		double GyroZ;	//Z轴角速度，单位°/s
		double AngleX;	//X轴角度，单位°
		double AngleY;	//Y轴角度，单位°
		double AngleZ;	//Z轴角度，单位°
		double Temper;	//温度，单位°C
	};

public:
	boolean begin(HardwareSerial* pSerialInterface, const double dAccX_Err = 0, const double dAccY_Err = 0, const double dAccZ_Err = 0, const double dGyroX_Err = 0, const double dGyroY_Err = 0, const double dGyroZ_Err = 0, const double dAngleX_Err = 0, const double dAngleY_Err = 0, const double dAngleZ_Err = 0);
	boolean begin(TwoWire* pI2CInterface,           const double dAccX_Err = 0, const double dAccY_Err = 0, const double dAccZ_Err = 0, const double dGyroX_Err = 0, const double dGyroY_Err = 0, const double dGyroZ_Err = 0);

	Data_Struct Data;
	boolean GetData(const boolean boolReversalX = false, const boolean boolReversalY = false, const boolean boolReversalZ = false);
};
//---------------------------------------------------------------------------
#endif
